@@ -42,9 +42,9 @@
 #'
 #' @section Interactions:
 #'
-#' You can interact a variable with a "factor-like" variable by using the syntax \code{var::fe(ref)}, where \code{fe} is the variable to be interacted with and the argument \code{ref} is a value of \code{fe} taken as a reference.
+#' You can interact a variable with a "factor-like" variable by using the syntax \code{var::fe(ref)}, where \code{fe} is the variable to be interacted with and the argument \code{ref} is a value (or several) of \code{fe} taken as a reference.
 #'
-#' The full syntax is: \code{var::fe(ref, confirm)}. You have two arguments, \code{ref} and \code{confirm}, the two are optional. The argument \code{confirm} is there to avoid mistakenly estimating a model with (too) many variables. If the variable \code{fe} takes over 100 different values, then you have to add the argument \code{confirm = TRUE}. It is important to note that *if you do not care about the standard-errors of the interactions*, then you can add interactions in the fixed-effects part of the formula (using the syntax fe[[var]], as explained in the section \dQuote{Varying slopes}).
+#' It is important to note that *if you do not care about the standard-errors of the interactions*, then you can add interactions in the fixed-effects part of the formula (using the syntax fe[[var]], as explained in the section \dQuote{Varying slopes}).
 #'
 #' Introducing interactions with this syntax leads to a different display of the interacted values in \code{\link[fixest]{etable}} and offers a special representation of the interacted coefficients in the function \code{\link[fixest]{coefplot}}. See examples.
 #'
@@ -229,6 +229,7 @@ feols = function(fml, data, weights, offset, panel.id, fixef, fixef.tol = 1e-6,
 
 		slope_flag = get("slope_flag", env)
 		slope_vars = get("slope_variables", env)
+
 		vars_demean <- cpp_demean(y, X, weights, iterMax = fixef.iter,
 		                          diffMax = fixef.tol, nb_cluster_all = fixef_sizes,
 		                          dum_vector = fixef_id_vector, tableCluster_vector = fixef_table_vector,
@@ -242,6 +243,22 @@ feols = function(fml, data, weights, offset, panel.id, fixef, fixef.tol = 1e-6,
 			res$means = vars_demean$means
 		}
 
+		if(any(slope_flag > 0) && any(res$iterations > 300)){
+		    # Maybe we have a convergence problem
+		    # This is poorly coded, but it's a temporary fix
+		    opt_fe = check_conv(y_demean, X_demean, fixef_id_vector, slope_flag, slope_vars, weights)
+
+		    # This is a bit too rough a check but it should catch the most problematic cases
+		    if(any(opt_fe > 1e-4)){
+		        msg = "There seems to be a convergence problem as regards the variables with varying slopes (in the RHS of your formula). The precision of the estimates may not be great. This is a known issue and there is work underway to solve it.\n As a workaround, you can use the variables with varying slopes as regular variables using the function interact (see ?interact)."
+		        if(fromGLM){
+		            res$warn_varying_slope = msg
+		        } else {
+		            warning(msg)
+		        }
+		    }
+		}
+
 		if(verbose >= 1){
 			if(length(fixef_sizes) > 1){
 				cat(" Demeaning in ", (proc.time() - time_demean)[3], "s (iter: ", paste0(c(tail(res$iterations, 1), res$iterations[-length(res$iterations)]), collapse = ", "), ")\n", sep="")
@@ -256,15 +273,27 @@ feols = function(fml, data, weights, offset, panel.id, fixef, fixef.tol = 1e-6,
 	# Estimation
 	if(!onlyFixef){
 
-		est = ols_fit(y_demean, X_demean, weights, correct_0w, nthreads)
+	    est = ols_fit(y_demean, X_demean, weights, correct_0w, nthreads)
+
+	    # Corner case: all variables not relevant
+	    if(!is.null(est$all_removed)){
+	        all_vars = colnames(X)
+
+	        if(isFixef){
+	            stop_up(ifsingle(all_vars, "The only variable ", "All variables"), enumerate_items(all_vars, "quote.is", nmax = 3), " collinear with the fixed effects. In such circumstances, the estimation is void.", up = fromGLM)
+	        } else {
+	            stop_up(ifsingle(all_vars, "The only variable ", "All variables"), enumerate_items(all_vars, "quote.is", nmax = 3), " virtually constant and equal to 0. In such circumstances, the estimation is void.", up = fromGLM)
+	        }
+	    }
 
 		# Formatting the result
-		coef = est$coefficients
-		names(coef) = colnames(X)
+	    coef = est$coefficients
+	    names(coef) = colnames(X)[!est$is_excluded]
 		res$coefficients = coef
 		# Additional stuff
 		res$residuals = est$residuals
 		res$multicol = est$multicol
+		if(fromGLM) res$is_excluded = est$is_excluded
 	} else {
 		res$residuals = y_demean
 		res$coefficients = coef = NULL
@@ -290,7 +319,31 @@ feols = function(fml, data, weights, offset, panel.id, fixef, fixef.tol = 1e-6,
 	# Post processing
 	#
 
+	# Collinearity message
+	collin.adj = 0
+	if(res$multicol){
+	    var_collinear = colnames(X)[est$is_excluded]
+	    if(notes){
+	        message(ifsingle(var_collinear, "The variable ", "Variables "), enumerate_items(var_collinear, "quote.has", nmax = 3), " been removed because of collinearity (see $collin.var).")
+	    }
+
+	    res$collin.var = var_collinear
+
+	    # full set of coeffficients with NAs
+	    collin.coef = setNames(rep(NA, ncol(X)), colnames(X))
+	    collin.coef[!est$is_excluded] = res$coefficients
+	    res$collin.coef = collin.coef
+
+	    if(isFixef){
+	        X = X[, !est$is_excluded, drop = FALSE]
+	    }
+	    X_demean = X_demean[, !est$is_excluded, drop = FALSE]
+
+	    collin.adj = sum(est$is_excluded)
+	}
+
 	n = length(y)
+	res$nparams = res$nparams - collin.adj
 	df_k = res$nparams
 	res$nobs = n
 
@@ -329,15 +382,16 @@ feols = function(fml, data, weights, offset, panel.id, fixef, fixef.tol = 1e-6,
 			res$sigma2 = cpp_ssq(res$residuals) / (length(y) - df_k)
 		}
 
-		if(est$multicol == TRUE){
-			if(warn){
-			    warning("Presence of collinearity, covariance not defined. Use function collinearity() to pinpoint the problems.")
-			    options("fixest_last_warning" = proc.time())
-			}
-			res$cov.unscaled = est$xwx_inv * NA
-		} else {
-			res$cov.unscaled = est$xwx_inv * res$sigma2
-		}
+		# if(est$multicol == TRUE){
+		# 	if(warn){
+		# 	    warning("Presence of collinearity, covariance not defined. Use function collinearity() to pinpoint the problems.")
+		# 	    options("fixest_last_warning" = proc.time())
+		# 	}
+		# 	res$cov.unscaled = est$xwx_inv * NA
+		# } else {
+		# 	res$cov.unscaled = est$xwx_inv * res$sigma2
+		# }
+		res$cov.unscaled = est$xwx_inv * res$sigma2
 
 		rownames(res$cov.unscaled) = colnames(res$cov.unscaled) = names(coef)
 
@@ -348,10 +402,10 @@ feols = function(fml, data, weights, offset, panel.id, fixef, fixef.tol = 1e-6,
 
 		# coeftable
 		zvalue <- coef/se
-		pvalue <- 2*pnorm(-abs(zvalue))
+		pvalue <- 2*pt(-abs(zvalue), max(n - df_k, 1))
 
-		coeftable <- data.frame("Estimate"=coef, "Std. Error"=se, "z value"=zvalue, "Pr(>|z|)"=pvalue)
-		names(coeftable) <- c("Estimate", "Std. Error", "z value",  "Pr(>|z|)")
+		coeftable <- data.frame("Estimate"=coef, "Std. Error"=se, "t value"=zvalue, "Pr(>|t|)"=pvalue)
+		names(coeftable) <- c("Estimate", "Std. Error", "t value",  "Pr(>|t|)")
 		row.names(coeftable) <- names(coef)
 
 		attr(se, "type") = attr(coeftable, "type") = "Standard"
@@ -391,41 +445,85 @@ feols = function(fml, data, weights, offset, panel.id, fixef, fixef.tol = 1e-6,
 	res
 }
 
-
 ols_fit = function(y, X, w, correct_0w = FALSE, nthreads){
-	# No control here -- done before
+    # No control here -- done before
 
-    if(correct_0w == FALSE){
-        # DEFAULT
-        xwx = cpppar_crossprod(X, w, nthreads)
-        xwy = cpppar_xwy(X, y, w, nthreads)
-    } else if(correct_0w == TRUE){
-        # to ignore 0w obs
-        is_0w = w == 0
-        xwx = cpppar_crossprod(X[!is_0w, , drop = FALSE], w[!is_0w], nthreads)
-        xwy = cpppar_xwy(X[!is_0w, , drop = FALSE], y[!is_0w], w[!is_0w], nthreads)
+    info_products = cpp_sparse_products(X, w, y, correct_0w, nthreads)
+    xwx = info_products$XtX
+    xwy = info_products$Xty
+
+    multicol = FALSE
+    info_inv = cpp_cholesky(xwx)
+
+    if(!is.null(info_inv$all_removed)){
+        # Means all variables are collinear! => can happen when using FEs
+        return(list(all_removed = TRUE))
     }
 
-	multicol = FALSE
-	xwx_inv = NULL
-	try(xwx_inv <- solve(xwx), silent = TRUE)
-	if(is.null(xwx_inv)){
-		multicol = TRUE
-		# if multicollinearity, we provide a solution anyway
-		# multicol is dealt with separately by the user
-		xwx_inv = MASS::ginv(xwx)
-	}
+    xwx_inv = info_inv$XtX_inv
+    is_excluded = info_inv$id_excl
 
-	beta = drop(xwx_inv %*% xwy)
+    multicol = any(is_excluded)
+
+    beta = as.vector(xwx_inv %*% xwy[!is_excluded])
+
+    fitted.values = cpppar_xbeta(X[, !is_excluded, drop = FALSE], beta, nthreads)
 
-    fitted.values = cpppar_xbeta(X, beta, nthreads)
     residuals = y - fitted.values
 
-	names(beta) = colnames(X)
+    res = list(xwx = xwx, coefficients = beta, fitted.values = fitted.values, xwx_inv = xwx_inv, multicol = multicol, residuals = residuals, is_excluded = is_excluded)
 
-	res = list(xwx = xwx, coefficients = beta, fitted.values = fitted.values, xwx_inv = xwx_inv, multicol = multicol, residuals = residuals)
+    res
+}
 
-	res
+
+
+check_conv = function(y, X, fixef_id_vector, slope_flag, slope_vars, weights){
+    # VERY SLOW!!!!
+    # IF THIS FUNCTION LASTS => TO BE PORTED TO C++
+
+    # y, X => variables that were demeaned
+
+    # For each variable: we compute the optimal FE coefficient
+    # it should be 0 if the algorithm converged
+
+    Q = length(slope_flag)
+
+    nobs = length(y)
+    if(length(X) == 1){
+        K = 1
+    } else {
+        K = NCOL(X) + 1
+    }
+
+    res = matrix(NA, K, Q)
+
+    for(k in 1:K){
+        if(k == 1){
+            x = y
+        } else {
+            x = X[, k - 1]
+        }
+
+        for(q in 1:Q){
+            index_id = 1:nobs + (q - 1) * nobs
+            fixef_id = fixef_id_vector[index_id]
+
+            if(slope_flag[q]){
+                index_var = 1:nobs + (cumsum(slope_flag)[q] - 1) * nobs
+                var = slope_vars[index_var]
+
+                num = tapply(weights * x * var, fixef_id, sum)
+                denom = tapply(weights * var^2, fixef_id, sum)
+                res[k, q] = max(abs(num/denom))
+
+            } else {
+                res[k, q] = max(abs(tapply(weights * x, fixef_id, mean)))
+            }
+        }
+    }
+
+    res
 }
 
 
@@ -467,7 +565,7 @@ ols_fit = function(y, X, w, correct_0w = FALSE, nthreads){
 #' \item{fixef_sizes}{The size of each fixed-effect (i.e. the number of unique identifierfor each fixed-effect dimension).}
 #' \item{y}{[where relevant] The dependent variable (used to compute the within-R2 when fixed-effects are present).}
 #' \item{convStatus}{Logical, convergence status of the IRWLS algorithm.}
-#' \item{weights_irls}{The weights of the last iteration of the IRWLS algorithm.}
+#' \item{irls_weights}{The weights of the last iteration of the IRWLS algorithm.}
 #' \item{obsRemoved}{[where relevant] In the case there were fixed-effects and some observations were removed because of only 0/1 outcome within a fixed-effect, it gives the row numbers of the observations that were removed. Also reports the NA observations that were removed.}
 #' \item{fixef_removed}{[where relevant] In the case there were fixed-effects and some observations were removed because of only 0/1 outcome within a fixed-effect, it gives the list (for each fixed-effect dimension) of the fixed-effect identifiers that were removed.}
 #' \item{coefficients}{The named vector of estimated coefficients.}
@@ -650,7 +748,7 @@ feglm.fit = function(y, X, fixef_mat, family = "poisson", offset, weights, start
     variance = family$variance
     linkfun = family$linkfun
     linkinv = family$linkinv
-    dev.resids = family$dev.resids
+    sum_dev.resids = family$sum_dev.resids
     valideta = family$valideta
     validmu = family$validmu
     mu.eta = family$mu.eta
@@ -708,7 +806,7 @@ feglm.fit = function(y, X, fixef_mat, family = "poisson", offset, weights, start
         } else {
             eta = offset_fe
             mu = linkinv(eta)
-            devold = dev.resids(y, mu, eta, wt = weights)
+            devold = sum_dev.resids(y, mu, eta, wt = weights)
         }
 
         wols_old = list(fitted.values = eta - offset)
@@ -725,7 +823,7 @@ feglm.fit = function(y, X, fixef_mat, family = "poisson", offset, weights, start
     if(init.type != "coef"){
         # starting deviance with constant equal to 1e-5
         # this is important for getting in step halving early (when deviance goes awry right from the start)
-        devold = dev.resids(y, rep(linkinv(1e-5), nobs), rep(1e-5, nobs), wt = weights)
+        devold = sum_dev.resids(y, rep(linkinv(1e-5), nobs), rep(1e-5, nobs), wt = weights)
         wols_old = list(fitted.values = rep(1e-5, nobs))
     }
 
@@ -804,7 +902,7 @@ feglm.fit = function(y, X, fixef_mat, family = "poisson", offset, weights, start
 
         mu = linkinv(eta)
 
-        dev = dev.resids(y, mu, eta, wt = weights)
+        dev = sum_dev.resids(y, mu, eta, wt = weights)
         dev_evol = dev - devold
 
         if(verbose >= 1) cat("Iteration: ", sprintf("%02i", iter), " -- Deviance = ", numberFormatNormal(dev), "\n", sep = "")
@@ -855,7 +953,7 @@ feglm.fit = function(y, X, fixef_mat, family = "poisson", offset, weights, start
                 eta_new = (eta_old + eta_new) / 2
 
                 mu = linkinv(eta_new + offset)
-                dev = dev.resids(y, mu, eta_new + offset, wt = weights)
+                dev = sum_dev.resids(y, mu, eta_new + offset, wt = weights)
                 dev_evol = dev - devold
 
                 if(verbose >= 3) cat("Step-halving: iter =", iter_sh, "-- dev:", numberFormatNormal(dev), "-- evol:", numberFormatNormal(dev_evol), "\n")
@@ -907,10 +1005,34 @@ feglm.fit = function(y, X, fixef_mat, family = "poisson", offset, weights, start
     #     warning_msg = paste(warning_msg, "Presence of collinearity. Use function collinearity() to pinpoint the problems.")
     # }
 
-    res$weights_irls = w # weights from the iteratively reweighted least square
+    # Collinearity message
+    collin.adj = 0
+    if(wols$multicol){
+        var_collinear = colnames(X)[wols$is_excluded]
+        if(notes) message(ifsingle(var_collinear, "The variable ", "Variables "), enumerate_items(var_collinear, "quote.has"), " been removed because of collinearity (see $collin.var).")
+
+        res$collin.var = var_collinear
+
+        # full set of coeffficients with NAs
+        collin.coef = setNames(rep(NA, ncol(X)), colnames(X))
+        collin.coef[!wols$is_excluded] = wols$coefficients
+        res$collin.coef = collin.coef
+
+        wols$X_demean = wols$X_demean[, !wols$is_excluded, drop = FALSE]
+        X = X[, !wols$is_excluded, drop = FALSE]
+
+        collin.adj = sum(wols$is_excluded)
+    }
+
+
+    res$irls_weights = w # weights from the iteratively reweighted least square
 
     res$coefficients = coef = wols$coefficients
 
+    if(!is.null(wols$warn_varying_slope)){
+        warning(wols$warn_varying_slope)
+    }
+
     res$linear.predictors = wols$fitted.values
     if(isOffset){
         res$linear.predictors = res$linear.predictors + offset
@@ -920,26 +1042,60 @@ feglm.fit = function(y, X, fixef_mat, family = "poisson", offset, weights, start
 
     if(onlyFixef) res$onlyFixef = onlyFixef
 
+    # dispersion + scores
+    if(family$family %in% c("poisson", "binomial")){
+        res$dispersion = 1
+    } else {
+        weighted_resids = wols$residuals * res$irls_weights
+        # res$dispersion = sum(weighted_resids ** 2) / sum(res$irls_weights)
+        # I use the second line to fit GLM's
+        res$dispersion = sum(weighted_resids * wols$residuals) / (res$nobs - res$nparams)
+    }
+
+    res$working_residuals = wols$residuals
+
     if(!onlyFixef && !lean){
         # score + hessian + vcov
-        res$scores = (wols$residuals * res$weights_irls) * wols$X_demean
-        # res$hessian = crossprod(res$X_demean * sqrt(res$weights_irls))
-        res$hessian = cpppar_crossprod(wols$X_demean, res$weights_irls, nthreads)
+
+        # dispersion + scores
+        if(family$family %in% c("poisson", "binomial")){
+            res$scores = (wols$residuals * res$irls_weights) * wols$X_demean
+            res$hessian = cpppar_crossprod(wols$X_demean, res$irls_weights, nthreads)
+        } else {
+            res$scores = (weighted_resids / res$dispersion) * wols$X_demean
+            res$hessian = cpppar_crossprod(wols$X_demean, res$irls_weights, nthreads) / res$dispersion
+        }
 
         # cov:
-        var <- NULL
-        try(var <- solve(res$hessian), silent = TRUE)
-        if(is.null(var) || wols$multicol){
-            if(is.null(var)){
-                warning_msg = paste(warning_msg, "Covariance not defined, presence of collinearity. Use function collinearity() to pinpoint the problems.")
-                res$cov.unscaled = res$hessian * NA
-            } else {
-                warning_msg = paste(warning_msg, "Presence of collinearity in the IRLS stage. Use function collinearity() to pinpoint the problems.")
-                res$cov.unscaled = var
-            }
-        } else {
-            res$cov.unscaled = var
+        # var <- NULL
+        # try(var <- solve(res$hessian), silent = TRUE)
+        # if(is.null(var) || wols$multicol){
+        #     if(is.null(var)){
+        #         warning_msg = paste(warning_msg, "Covariance not defined, presence of collinearity. Use function collinearity() to pinpoint the problems.")
+        #         res$cov.unscaled = res$hessian * NA
+        #     } else {
+        #         warning_msg = paste(warning_msg, "Presence of collinearity in the IRLS stage. Use function collinearity() to pinpoint the problems.")
+        #         res$cov.unscaled = var
+        #     }
+        # } else {
+        #     res$cov.unscaled = var
+        # }
+        info_inv = cpp_cholesky(res$hessian, nthreads)
+        if(!is.null(info_inv$all_removed)){
+            # This should not occur, but I prefer to be safe
+            stop("Not any single variable with a positive variance was found after the weighted-OLS stage. (If possible, could you send a replicable example to fixest's author? He's curious about when that actually happens, since in theory it should never happen.)")
         }
+
+        var = info_inv$XtX_inv
+        is_excluded = info_inv$id_excl
+
+        if(any(is_excluded)){
+            # There should be no remaining collinearity
+            warning_msg = paste(warning_msg, "Residual collinearity was found after the weighted-OLS stage. The covariance is not defined. (This should not happen. If possible, could you send a replicable example to fixest's author? He's curious about when that actually happen.)")
+            var = matrix(NA, length(is_excluded), length(is_excluded))
+        }
+        res$cov.unscaled = var
+
         rownames(res$cov.unscaled) = colnames(res$cov.unscaled) = names(coef)
 
         # se
@@ -949,10 +1105,17 @@ feglm.fit = function(y, X, fixef_mat, family = "poisson", offset, weights, start
 
         # coeftable
         zvalue <- coef/se
-        pvalue <- 2*pnorm(-abs(zvalue))
+        use_t = !family$family %in% c("poisson", "binomial")
+        if(use_t){
+            pvalue <- 2*pt(-abs(zvalue), max(res$nobs - res$nparams, 1))
+            ctable_names = c("Estimate", "Std. Error", "t value",  "Pr(>|t|)")
+        } else {
+            pvalue <- 2*pnorm(-abs(zvalue))
+            ctable_names = c("Estimate", "Std. Error", "z value",  "Pr(>|z|)")
+        }
 
         coeftable <- data.frame("Estimate"=coef, "Std. Error"=se, "z value"=zvalue, "Pr(>|z|)"=pvalue)
-        names(coeftable) <- c("Estimate", "Std. Error", "z value",  "Pr(>|z|)")
+        names(coeftable) <- ctable_names
         row.names(coeftable) <- names(coef)
 
         attr(se, "type") = attr(coeftable, "type") = "Standard"
@@ -970,6 +1133,7 @@ feglm.fit = function(y, X, fixef_mat, family = "poisson", offset, weights, start
     n = length(y)
     res$nobs = n
 
+    res$nparams = res$nparams - collin.adj
     df_k = res$nparams
 
     # r2s
@@ -1665,7 +1829,7 @@ feNmlm = function(fml, data, family=c("poisson", "negbin", "logit", "gaussian"),
 		theta = coef[".theta"]
 		res$theta = theta
 
-		if(theta > 1000){
+		if(notes && theta > 1000){
 			message("Very high value of theta (", theta, "). There is no sign of overdisperion, you may consider a Poisson model.")
 		}
 
@@ -1744,8 +1908,8 @@ format_error_msg = function(x, origin){
 
     x = gsub("\n+$", "", x)
 
-    if(grepl("^Error (in|:) (fe|fixest)[^\n]+\n", x)){
-        res = gsub("^Error (in|:) (fe|fixest)[^\n]+\n *(.+)", "\\3", x)
+    if(grepl("^Error (in|:|: in) (fe|fixest)[^\n]+\n", x)){
+        res = gsub("^Error (in|:|: in) (fe|fixest)[^\n]+\n *(.+)", "\\3", x)
     } else if(grepl("[Oo]bject '.+' not found", x)) {
         res = x
     } else {
